"""
Base class for Bedrock foundation models
"""

import logging
from typing import Iterator, Literal

import boto3
from pydantic import BaseModel, Field

from fence.models.base import LLM, InvokeMixin, MessagesMixin, StreamMixin
from fence.templates.messages import Messages

logger = logging.getLogger(__name__)

##########
# Models #
##########


class InferenceConfig(BaseModel):
    max_tokens: int | None = Field(
        None,
        ge=1,
        description="The maximum number of tokens to allow in the generated response. "
        "The default value is the maximum allowed value for the model that you are using.",
    )
    stop_sequences: list[str] | None = Field(
        None,
        min_items=0,
        max_items=4,
        description="A list of stop sequences. A stop sequence is a sequence of characters that causes the model to stop generating the response.",
    )
    temperature: float | None = Field(
        None,
        ge=0,
        le=1,
        description="The likelihood of the model selecting higher-probability options while generating a response. "
        "A lower value makes the model more likely to choose higher-probability options, while a higher value "
        "makes the model more likely to choose lower-probability options.",
    )
    top_p: float | None = Field(
        None,
        ge=0,
        le=1,
        description="The percentage of most-likely candidates that the model considers for the next token. "
        "For example, if you choose a value of 0.8 for topP, the model selects from the top 80% of the probability "
        "distribution of tokens that could be next in the sequence.",
    )


class JSONSchema(BaseModel):
    type: Literal["object"]
    properties: dict[str, dict[str, str]]
    required: list[str]


class ToolInputSchema(BaseModel):
    json_field: JSONSchema = Field(
        ..., alias="json", description="The input schema for the tool in JSON format."
    )


class ToolSpec(BaseModel):
    name: str = Field(
        ..., min_length=1, max_length=64, description="The name for the tool."
    )
    description: str | None = Field(
        None, min_length=1, description="The description for the tool."
    )
    inputSchema: ToolInputSchema = Field(
        ..., description="The input schema for the tool in JSON format."
    )


class Tool(BaseModel):
    toolSpec: ToolSpec | None = Field(..., description="The tool specification.")


class ToolConfig(BaseModel):
    tools: list[Tool] = Field(
        ..., min_length=1, description="The tools to use for the conversation."
    )


############################################
# Base class for Bedrock foundation models #
############################################


class BedrockBase(LLM, MessagesMixin, StreamMixin, InvokeMixin):
    """Base class for Bedrock foundation models"""

    inference_type = "bedrock"

    ###############
    # Constructor #
    ###############

    def __init__(
        self,
        source: str | None = None,
        inferenceConfig: InferenceConfig | dict | None = None,
        toolConfig: ToolConfig | dict | None = None,
        full_response: bool = False,
        **kwargs,
    ):
        """
        Initialize a Bedrock model

        :param str source: An indicator of where (e.g., which feature) the model is operating from. Useful to pass to the logging callback
        :param **kwargs: Additional keyword arguments
        """
        super().__init__(**kwargs)
        self.source = source

        # Config
        self.inferenceConfig = (
            InferenceConfig(**inferenceConfig)
            if isinstance(inferenceConfig, dict)
            else inferenceConfig
        )
        self.toolConfig = (
            ToolConfig(**toolConfig) if isinstance(toolConfig, dict) else toolConfig
        )
        self.full_response = full_response

        # AWS
        self.region = kwargs.get("region", "eu-central-1")
        self.client = boto3.client("bedrock-runtime", self.region)

    def invoke(self, prompt: str | Messages, **kwargs) -> str:
        """
        Call the model with the given prompt
        :param prompt: text to feed the model
        :param **kwargs: Additional keyword arguments to override the default model kwargs
        :return: response
        """
        if "full_response" in kwargs:
            self.full_response = kwargs["full_response"]
        return self._invoke(prompt=prompt, stream=False, **kwargs)

    def stream(self, prompt: str | Messages, **kwargs) -> Iterator[str]:
        """
        Stream the model response with the given prompt.
        :param prompt: text to feed the model
        :param **kwargs: Additional keyword arguments to override the default model kwargs
        :return: stream of responses
        """
        if "full_response" in kwargs:
            self.full_response = kwargs["full_response"]
        yield from self._invoke(prompt=prompt, stream=True, **kwargs)

    def _invoke(self, prompt: str | Messages, stream: bool, **kwargs):
        """
        Centralized method to handle both invoke and stream.
        """
        # Check if the prompt is valid
        self._check_if_prompt_is_valid(prompt=prompt)

        # Prepare the request parameters with override kwargs
        invoke_params = self._prepare_request(prompt)

        # Invoke the model
        if stream:
            return self._handle_stream(invoke_params=invoke_params)
        else:
            return self._handle_invoke(invoke_params=invoke_params)

    def _prepare_request(self, prompt: str | Messages) -> dict:
        """
        Prepares the request parameters for the Bedrock API.

        :param prompt: text or Messages object to format
        :return: request parameters dictionary
        """
        messages = self._format_prompt(prompt)

        # Prepare the request parameters
        invoke_params = {
            "modelId": self.model_id,
            "messages": messages["messages"],
        }
        if self.inferenceConfig:
            invoke_params["inferenceConfig"] = self.inferenceConfig.model_dump()
        if self.toolConfig:
            invoke_params["toolConfig"] = self.toolConfig.model_dump(by_alias=True)
        if "system" in messages:
            invoke_params["system"] = messages["system"]
        return invoke_params

    def _handle_invoke(self, invoke_params: dict) -> str | dict:
        """
        Handles synchronous invocation.

        :param invoke_params: parameters for the Bedrock API
        :return: completion text or full response object if full_response is True
        """
        try:
            # Call the Bedrock API
            response = self.client.converse(**invoke_params)

            # Extract and log metrics regardless of full_response setting
            metrics = self._extract_metrics(response.get("usage", {}))
            self._log_callback(**metrics)

            # Return the full response if requested, without processing
            if self.full_response:
                return response

            # Process the response for completion text
            completion = self._process_response(response)
            return completion

        except Exception as e:
            raise ValueError(f"Error raised by bedrock service: {e}")

    def _handle_stream(self, invoke_params: dict) -> Iterator[str | dict]:
        """
        Handles streaming invocation.

        :param invoke_params: parameters for the Bedrock API
        :return: stream of responses or full response objects if full_response is True
        """
        try:
            # Call the Bedrock streaming API
            response = self.client.converse_stream(**invoke_params)

            # Process the streaming response
            stream = response.get("stream")
            if not stream:
                raise ValueError("No stream found in response")

            # Stream the response
            for event in stream:
                # If full_response is True, yield the entire event
                if self.full_response:
                    yield event
                    continue

                # If the event is a message start, print the role
                if "messageStart" in event:
                    logger.debug(
                        f"Message started. Role: {event['messageStart']['role']}"
                    )

                # Handle content blocks - both text and tool calls
                if "contentBlockDelta" in event:
                    delta = event["contentBlockDelta"]["delta"]
                    if "text" in delta:
                        text = delta["text"]
                        logger.debug(f"Streaming chunk: {text}")
                        yield text
                    elif "toolCall" in delta:
                        tool_call = delta["toolCall"]
                        logger.debug(f"Tool call: {tool_call}")
                        yield str(tool_call)

                # If the event is a message stop, print the stop reason
                if "messageStop" in event:
                    logger.debug(
                        f"Message stopped. Reason: {event['messageStop']['stopReason']}"
                    )

                if "metadata" in event:
                    self._log_callback(**self._extract_metrics(event["metadata"]))

        except Exception as e:
            raise ValueError(f"Error in streaming response from Bedrock service: {e}")

    def _process_response(self, response: dict) -> str:
        """
        Extracts completion text from the response.
        """
        response_body = response.get("output", {}).get("message", {})
        completion = response_body.get("content", [{}])[0].get("text", "")
        return completion

    def _extract_metrics(self, metadata: dict) -> dict:
        """
        Extracts relevant metrics from the metadata.
        """
        return {
            "input_token_count": metadata.get("inputTokens", 0),
            "output_token_count": metadata.get("outputTokens", 0),
            "total_token_count": metadata.get("totalTokens", 0),
            "latency_ms": metadata.get("metrics", {}).get("latencyMs"),
        }

    def _format_prompt(self, prompt: str | Messages) -> dict:
        """
        Format the prompt for the Bedrock API

        :param prompt: text or Messages object to format
        :return: formatted messages dictionary
        """
        # Format prompt: Claude3 models expect a list of messages, with content, roles, etc.
        # However, if we receive a string, we will format it as a single user message for ease of use.
        if isinstance(prompt, Messages):
            messages = prompt.model_dump_bedrock_converse()
            # Strip `type` key from each message
            messages["messages"] = [
                {k: v for k, v in message.items() if k != "type"}
                for message in messages["messages"]
            ]
        elif isinstance(prompt, str):
            messages = {
                "messages": [
                    {
                        "role": "user",
                        "content": [
                            {
                                "text": prompt,
                            }
                        ],
                    }
                ]
            }
        else:
            raise ValueError("Prompt must be a string or a list of messages")
        return messages


if __name__ == "__main__":
    tool_config = {
        "tools": [
            {
                "toolSpec": {
                    "name": "top_song",
                    "description": "Get the most popular song played on a radio station.",
                    "inputSchema": {
                        "json": {
                            "type": "object",
                            "properties": {
                                "sign": {
                                    "type": "string",
                                    "description": "The call sign for the radio station for which you want the most popular song. Example calls signs are WZPZ, and WKRP.",
                                }
                            },
                            "required": ["sign"],
                        }
                    },
                }
            }
        ]
    }
    # Try to parse the tool config
    try:
        tool_config = ToolConfig(**tool_config)
    except Exception as e:
        print(e)



import os
import pytest
from unittest.mock import Mock, patch
import json
import requests

from fence.models.mistral.mistral import Mistral, MistralBase
from fence.templates.messages import Message, Messages, MessagesTemplate

@pytest.fixture
def mock_api_response():
    return {
        "id": "test_id",
        "object": "chat.completion",
        "created": 1234567890,
        "model": "mistral-large-latest",
        "choices": [
            {
                "index": 0,
                "message": {
                    "role": "assistant",
                    "content": "Test response"
                },
                "finish_reason": "stop"
            }
        ],
        "usage": {
            "prompt_tokens": 10,
            "completion_tokens": 5,
            "total_tokens": 15
        }
    }

@pytest.fixture
def mock_response(mock_api_response):
    response = Mock()
    response.status_code = 200
    response.json.return_value = mock_api_response
    return response

@pytest.fixture
def mistral_model():
    with patch.dict(os.environ, {"MISTRAL_API_KEY": "test_key"}):
        return Mistral(model_id="mistral-large-latest")

def test_mistral_initialization():
    """Test Mistral model initialization"""
    # Test with API key in environment
    with patch.dict(os.environ, {"MISTRAL_API_KEY": "test_key"}):
        model = Mistral(model_id="mistral-large-latest")
        assert model.model_id == "mistral-large-latest"
        assert model.model_name == "Mistral-large-latest"
        assert model.api_key == "test_key"
        assert model.url == "https://api.mistral.ai"
        assert model.chat_endpoint == "https://api.mistral.ai/v1/chat/completions"

    # Test with API key as parameter
    model = Mistral(model_id="mistral-large-latest", api_key="direct_key")
    assert model.api_key == "direct_key"

    # Test missing API key
    with pytest.raises(ValueError):
        with patch.dict(os.environ, {}, clear=True):
            Mistral(model_id="mistral-large-latest")

def test_mistral_model_parameters(mistral_model):
    """Test model parameters initialization"""
    assert mistral_model.model_kwargs["top_p"] == 1
    assert mistral_model.model_kwargs["temperature"] is None
    assert mistral_model.model_kwargs["max_tokens"] is None
    assert mistral_model.model_kwargs["stop"] is None

    # Test with custom parameters
    model = Mistral(
        model_id="mistral-large-latest",
        api_key="test_key",
        temperature=0.7,
        max_tokens=100,
        top_p=0.9,
        stop=["END"]
    )
    assert model.model_kwargs["temperature"] == 0.7
    assert model.model_kwargs["max_tokens"] == 100
    assert model.model_kwargs["top_p"] == 0.9
    assert model.model_kwargs["stop"] == ["END"]

@patch('requests.post')
def test_invoke_with_string_prompt(mock_post, mistral_model, mock_response, mock_api_response):
    """Test model invocation with string prompt"""
    mock_post.return_value = mock_response
    
    response = mistral_model.invoke("Test prompt")
    
    assert response == "Test response"
    mock_post.assert_called_once()
    
    # Verify request format
    call_args = mock_post.call_args
    headers = call_args.kwargs['headers']
    data = json.loads(call_args.kwargs['data'])
    
    assert headers['Authorization'] == 'Bearer test_key'
    assert headers['Content-Type'] == 'application/json'
    assert data['messages'][0]['role'] == 'user'
    assert data['messages'][0]['content'] == 'Test prompt'

@patch('requests.post')
def test_invoke_with_messages(mock_post, mistral_model, mock_response):
    """Test model invocation with Messages object"""
    mock_post.return_value = mock_response
    
    messages = Messages(
        system="System prompt",
        messages=[
            Message(role="user", content="User message"),
            Message(role="assistant", content="Assistant message")
        ]
    )
    
    response = mistral_model.invoke(messages)
    
    assert response == "Test response"
    mock_post.assert_called_once()
    
    # Verify request format
    call_args = mock_post.call_args
    data = json.loads(call_args.kwargs['data'])
    assert len(data['messages']) == 3
    assert data['messages'][0]['role'] == 'system'
    assert data['messages'][0]['content'] == 'System prompt'

@patch('requests.post')
def test_invoke_with_template(mock_post, mistral_model, mock_response):
    """Test model invocation with MessagesTemplate"""
    mock_post.return_value = mock_response
    
    messages = Messages(
        system="Respond in a {tone} tone",
        messages=[Message(role="user", content="Why is the sky {color}?")]
    )
    template = MessagesTemplate(source=messages)
    
    response = mistral_model.invoke(template.render(tone="sarcastic", color="blue"))
    
    assert response == "Test response"
    mock_post.assert_called_once()
    
    # Verify request format
    call_args = mock_post.call_args
    data = json.loads(call_args.kwargs['data'])
    assert data['messages'][0]['content'] == 'Respond in a sarcastic tone'
    assert data['messages'][1]['content'] == 'Why is the sky blue?'

@patch('requests.post')
def test_error_handling(mock_post, mistral_model):
    """Test error handling"""
    # Test API error response
    error_response = Mock()
    error_response.status_code = 400
    error_response.text = "Bad Request"
    mock_post.return_value = error_response
    
    with pytest.raises(ValueError) as exc_info:
        mistral_model.invoke("Test prompt")
    assert "Error raised by Mistral service" in str(exc_info.value)
    
    # Test network error
    mock_post.side_effect = requests.exceptions.RequestException("Network error")
    with pytest.raises(ValueError) as exc_info:
        mistral_model.invoke("Test prompt")
    assert "Error raised by Mistral service" in str(exc_info.value)

def test_invalid_prompt_types(mistral_model):
    """Test handling of invalid prompt types"""
    with pytest.raises(ValueError):
        mistral_model.invoke(None)
    
    with pytest.raises(ValueError):
        mistral_model.invoke("")
    
    with pytest.raises(ValueError):
        mistral_model.invoke(123)

@patch('requests.post')
def test_metrics_logging(mock_post, mistral_model, mock_response):
    """Test metrics logging functionality"""
    mock_post.return_value = mock_response
    
    # Add a mock logging callback
    mock_callback = Mock()
    mistral_model._log_callback = mock_callback
    
    mistral_model.invoke("Test prompt")
    
    # Verify metrics logging
    mock_callback.assert_called_once()
    metrics = mock_callback.call_args[1]
    assert metrics["input_token_count"] == 10
    assert metrics["output_token_count"] == 5
    assert metrics["input_word_count"] == 2  # "Test prompt"
    assert metrics["output_word_count"] == 2  # "Test response" 


import pytest

from fence.models.base import Messages
from fence.models.bedrock.base import BedrockBase


# Create a mock model class that inherits from BedrockBase
class MockBedrockModel(BedrockBase):
    """Mock Bedrock model for testing"""

    def __init__(self, **kwargs):
        """
        Initialize mock model with a predefined model_id
        :param kwargs: Additional keyword arguments
        """
        super().__init__(**kwargs)
        self.model_id = "MODEL_ID_HAIKU"
        self.inferenceConfig = {
            "temperature": 1,
            "max_tokens": None,
        }
        self.toolConfig = kwargs.get(
            "toolConfig",
            {
                "stream": False,
            },
        )

    def _invoke(self, prompt: str | Messages, stream=False, **kwargs) -> str | dict:
        """
        Override _invoke to simulate a successful response without actual AWS call
        :param prompt: The input prompt
        :param stream: Whether to stream the response
        :param kwargs: Additional keyword arguments
        :return: The mock response text or full response object
        """
        self._check_if_prompt_is_valid(prompt)

        # Simulate tool call response if toolConfig is present
        if self.toolConfig and "tools" in self.toolConfig:
            if self.full_response:
                if stream:
                    return [
                        {
                            "messageStart": {"role": "assistant"},
                        },
                        {
                            "contentBlockDelta": {
                                "delta": {
                                    "toolCall": {
                                        "name": "top_song",
                                        "arguments": {"sign": "WABC"},
                                    }
                                }
                            }
                        },
                        {"messageStop": {"stopReason": "tool_call"}},
                    ]
                return {
                    "output": {
                        "message": {
                            "content": [
                                {
                                    "toolCall": {
                                        "name": "top_song",
                                        "arguments": {"sign": "WABC"},
                                    }
                                }
                            ]
                        }
                    },
                    "usage": {"inputTokens": 10, "outputTokens": 20},
                }
            if stream:
                return ["I'll check the top song for WABC."]
            return "I'll check the top song for WABC."

        # Regular text response
        if self.full_response:
            if stream:
                return [
                    {
                        "messageStart": {"role": "assistant"},
                    },
                    {
                        "contentBlockDelta": {"delta": {"text": "Test response"}},
                    },
                    {"messageStop": {"stopReason": "end_turn"}},
                ]
            return {
                "output": {"message": {"content": [{"text": "Test response"}]}},
                "usage": {"inputTokens": 10, "outputTokens": 20},
            }
        if stream:
            return ["Test response"]
        return "Test response"


class MockBedrockModelNoConfig(BedrockBase):
    """Mock Bedrock model for testing without configs"""

    def __init__(self, **kwargs):
        """
        Initialize mock model with a predefined model_id and no configs
        :param kwargs: Additional keyword arguments
        """
        super().__init__(**kwargs)
        self.model_id = "MODEL_ID_HAIKU"


class TestBedrockBase:
    @pytest.fixture
    def bedrock_base(self):
        """Create a MockBedrockModel instance for testing"""
        return MockBedrockModel(
            source="test_source",
            full_response=False,
            metric_prefix="test_prefix",
            extra_tags={"test_tag": "value"},
        )

    @pytest.fixture
    def bedrock_base_full_response(self):
        """Create a MockBedrockModel instance with full_response=True for testing"""
        return MockBedrockModel(
            source="test_source",
            full_response=True,
            metric_prefix="test_prefix",
            extra_tags={"test_tag": "value"},
        )

    @pytest.fixture
    def bedrock_base_no_config(self):
        """Create a MockBedrockModelNoConfig instance for testing"""
        return MockBedrockModelNoConfig(
            source="test_source",
            full_response=False,
            metric_prefix="test_prefix",
            extra_tags={"test_tag": "value"},
        )

    @pytest.fixture
    def bedrock_base_with_tools(self):
        """Create a MockBedrockModel instance with tools for testing"""
        return MockBedrockModel(
            source="test_source",
            full_response=False,
            metric_prefix="test_prefix",
            extra_tags={"test_tag": "value"},
            toolConfig={
                "tools": [
                    {
                        "toolSpec": {
                            "name": "top_song",
                            "description": "Get the most popular song played on a radio station.",
                            "inputSchema": {
                                "json": {
                                    "type": "object",
                                    "properties": {
                                        "sign": {
                                            "type": "string",
                                            "description": "The call sign for the radio station",
                                        }
                                    },
                                    "required": ["sign"],
                                }
                            },
                        }
                    }
                ]
            },
        )

    @pytest.fixture
    def bedrock_base_with_tools_full_response(self):
        """Create a MockBedrockModel instance with tools and full_response=True for testing"""
        return MockBedrockModel(
            source="test_source",
            full_response=True,
            metric_prefix="test_prefix",
            extra_tags={"test_tag": "value"},
            toolConfig={
                "tools": [
                    {
                        "toolSpec": {
                            "name": "top_song",
                            "description": "Get the most popular song played on a radio station.",
                            "inputSchema": {
                                "json": {
                                    "type": "object",
                                    "properties": {
                                        "sign": {
                                            "type": "string",
                                            "description": "The call sign for the radio station",
                                        }
                                    },
                                    "required": ["sign"],
                                }
                            },
                        }
                    }
                ]
            },
        )

    def test_initialization(self, bedrock_base):
        """Test that the BedrockBase is initialized correctly"""
        assert bedrock_base.source == "test_source"
        assert bedrock_base.metric_prefix == "test_prefix"
        assert bedrock_base.logging_tags == {"test_tag": "value"}
        assert bedrock_base.model_id == "MODEL_ID_HAIKU"

        # Check default config values
        assert bedrock_base.inferenceConfig == {
            "temperature": 1,
            "max_tokens": None,
        }
        assert bedrock_base.toolConfig == {
            "stream": False,
        }

    def test_initialization_without_configs(self, bedrock_base_no_config):
        """Test that the BedrockBase is initialized correctly without configs"""
        assert bedrock_base_no_config.source == "test_source"
        assert bedrock_base_no_config.metric_prefix == "test_prefix"
        assert bedrock_base_no_config.logging_tags == {"test_tag": "value"}
        assert bedrock_base_no_config.model_id == "MODEL_ID_HAIKU"

        # Check that configs are None when not provided
        assert bedrock_base_no_config.inferenceConfig is None
        assert bedrock_base_no_config.toolConfig is None

    def test_invoke_with_different_prompt_types(self, bedrock_base):
        """Test invoking with different prompt types"""
        # Test with string prompt
        str_response = bedrock_base.invoke("Test prompt")
        assert str_response == "Test response"

        # Test with Messages prompt
        messages = Messages(messages=[{"role": "user", "content": "Test prompt"}])
        msg_response = bedrock_base.invoke(messages)
        assert msg_response == "Test response"

    def test_invoke_with_override_kwargs(self, bedrock_base):
        """Test invoking with override kwargs"""
        response = bedrock_base.invoke("Test prompt", temperature=0.7, maxTokens=1024)
        assert response == "Test response"

    def test_invalid_prompt_type(self, bedrock_base):
        """Test invoking with an invalid prompt type"""
        with pytest.raises(
            ValueError, match="Prompt must be a string or a Messages object!"
        ):
            print(bedrock_base.invoke(123))  # Invalid type

    def test_invoke_with_full_response(self, bedrock_base_full_response):
        """Test invoking with full_response=True"""
        response = bedrock_base_full_response.invoke("Test prompt")
        assert isinstance(response, dict)
        assert response["output"]["message"]["content"][0]["text"] == "Test response"
        assert response["usage"]["inputTokens"] == 10
        assert response["usage"]["outputTokens"] == 20

    def test_invoke_with_tools(self, bedrock_base_with_tools):
        """Test invoking with tools"""
        response = bedrock_base_with_tools.invoke("Test prompt")
        assert response == "I'll check the top song for WABC."

    def test_invoke_with_tools_full_response(
        self, bedrock_base_with_tools_full_response
    ):
        """Test invoking with tools and full_response=True"""
        response = bedrock_base_with_tools_full_response.invoke("Test prompt")
        assert isinstance(response, dict)
        assert (
            response["output"]["message"]["content"][0]["toolCall"]["name"]
            == "top_song"
        )
        assert (
            response["output"]["message"]["content"][0]["toolCall"]["arguments"]["sign"]
            == "WABC"
        )
        assert response["usage"]["inputTokens"] == 10
        assert response["usage"]["outputTokens"] == 20

    def test_stream_with_tools(self, bedrock_base_with_tools):
        """Test streaming with tools"""
        chunks = list(bedrock_base_with_tools.stream("Test prompt"))
        assert len(chunks) == 1
        assert chunks[0] == "I'll check the top song for WABC."

    def test_stream_with_tools_full_response(
        self, bedrock_base_with_tools_full_response
    ):
        """Test streaming with tools and full_response=True"""
        chunks = list(bedrock_base_with_tools_full_response.stream("Test prompt"))
        assert len(chunks) == 3  # messageStart, contentBlockDelta, messageStop
        assert chunks[0]["messageStart"]["role"] == "assistant"
        assert chunks[1]["contentBlockDelta"]["delta"]["toolCall"]["name"] == "top_song"
        assert chunks[2]["messageStop"]["stopReason"] == "tool_call"

    def test_stream_without_tools(self, bedrock_base):
        """Test streaming without tools"""
        chunks = list(bedrock_base.stream("Test prompt"))
        assert len(chunks) == 1
        assert chunks[0] == "Test response"

    def test_stream_without_tools_full_response(self, bedrock_base_full_response):
        """Test streaming without tools and full_response=True"""
        chunks = list(bedrock_base_full_response.stream("Test prompt"))
        assert len(chunks) == 3  # messageStart, contentBlockDelta, messageStop
        assert chunks[0]["messageStart"]["role"] == "assistant"
        assert chunks[1]["contentBlockDelta"]["delta"]["text"] == "Test response"
        assert chunks[2]["messageStop"]["stopReason"] == "end_turn"
